/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */
#define FORBIDDEN_SYMBOL_EXCEPTION_time_h
#include "common/scummsys.h"

#if !defined(DISABLE_DEFAULT_EVENTMANAGER)


#include "common/system.h"
#include "common/config-manager.h"
#include "common/translation.h"
#include "backends/events/default/default-events.h"
#include "backends/keymapper/keymapper.h"
#include "backends/keymapper/remap-dialog.h"
#include "backends/vkeybd/virtual-keyboard.h"
#include "backends/platform/android/jni.h"

#include "engines/engine.h"
#include "gui/message.h"

#include "backends/platform/android/define_gob.h"
#include "backends/platform/android/android_common.h"
#include "backends/platform/android/button.h"
#include "backends/platform/android/textbutton.h"
//#include "backends/platform/android/hintview.h"
#include "backends/platform/android/saveloadview.h"

#include <time.h>

extern TextButton back;
extern TextButton moveTo;
extern Button onOff;
extern Button action;
extern Button menuButton;
extern Button skip;
extern Button gameovercancel;
extern Button gameoverok;
extern Button switchgob;
extern TextButton leave;
extern TextButton resume;
extern TextButton restart;
extern TextButton save;
extern TextButton load;
extern Button endlvlok;
extern TextButton settings;
extern TextButton extras;
extern TextButton journal;
extern TextButton hint;
int loadLoop;
int saveLoop;
//extern TextButton test_textbutton;
bool sendEventAction;


extern unsigned int numSlot;
bool bStartWait=false;
extern bool bSimu;

extern bool bIntro;
bool bExtras = false;
bool bSave=false;
bool enable_pause;
int slotSave;
int _mouseX;
int _mouseY;
bool lockNext;
bool bWaitEnd=false;
bool StartSimu=false;
extern Common::Rect game_screen;

extern bool bBagOn;
extern bool bShowSkipButton;
bool SimuMoveTo;
bool SimuJournal;
bool bNoUpdateHotspot=false;
extern int counter;
bool bSkipCredits=false;

extern bool WarningMessage;

//extern HintView hintView;
extern SaveLoadView saveLoadView;

extern bool bLoaded;
extern int numGobliiin;
extern const char* niveau;
extern bool bSimulCode;
extern bool bSimulation;
extern int bCptSimul;
extern bool bLvLOneLoaded;
extern int etat;
int etatSimulation=-1;
extern int cptEsc;
extern int bLoop2;
extern int bLoop3;
extern int bLoop;
extern int cpt;
extern int idLvlToLoad;
extern int idLvlCourant;
extern int bLoading;
extern bool bSaving;
extern int gobAction;
extern bool bPause;
extern bool bInGame;
extern bool bSkip;
bool bSkipIntro=false;
extern int currentGame;
extern bool bRestart;
extern Common::Point posSimulation;
extern bool bReload;
extern bool bExit;
extern bool bGameOverOn;
extern bool bSimulGO;
extern bool bSimulGOver;
extern bool bGoHotspot;
extern int bGo;
extern bool bResume;
extern bool bGoSim;
extern int saveEtat;
extern int loadEtat;

extern uint16 secondCharactTop;
extern uint16 secondCharactLeft;
extern uint16 secondCharactRight;
extern uint16 secondCharactBottom;
extern bool secondCharactEnable;
extern uint16 firstCharactTop;
extern uint16 firstCharactLeft;
extern uint16 firstCharactRight;
extern uint16 firstCharactBottom;
extern bool firstCharactEnable;

extern bool fingusBusy;
extern bool winkleBusy;
extern int fingusLeft;
extern int fingusTop;
extern int winkleLeft;
extern int winkleTop;
extern int gob3State;
extern bool changeGobliin;
extern int numberGobliin;
extern int princeLeft;
extern int princeTop;
extern int princeLastLeft;
extern Common::Point oldPos;
extern bool fakePos;
extern bool bFakeMousePause;
extern bool Top;
int num=0;
bool next=false;
int kTempo=15;
bool bMoveTo;
bool bResetAction;
bool charactereBusy=false;

extern int16 myHackSpotT;
extern int16 myHackSpotL;
extern Common::Point prePos;
bool bResetActionMoveTo;

bool showHotspot;
bool sendEvent;
bool nextGob;

int kTempoSwitch;
int posx;
int posy;

extern bool bLoadingIntro;


template<class T>
static inline T scalef(T in, float numerator, float denominator) {
	return static_cast<float>(in) * numerator / denominator;
}


void clipMouse(Common::Point &p) {
	p.x = CLIP(p.x, int16(0), int16(320 - 1));
	
	p.y = CLIP(p.y, int16(0), int16(200 - 1));
}

void scaleMouse(Common::Point &p, int x, int y) {
	x -= game_screen.left;
	y -= game_screen.top;
	p.x = scalef(x, 320, game_screen.width());
	p.y = scalef(y, 200, game_screen.height());
}

bool DefaultEventManager::screen_contains(Common::Point point,Common::Point &old){
	debugdot("game contain it ?");
	if(bExit)
		return true;
	if(game_screen.contains(point.x,point.y) ){
		debugdot("tap on screen");
		
		if(bPause || bExtras || bGameOverOn || bGoHotspot || /*hintView.isEnable() ||*/ bSimu || bSimulation){
			
			//posx=point.x;posy=point.y;
			return false;
		}else {	
			if(point.y<=ceil((2*game_screen.height()/200)) && currentGame!=1)
				point.y=3*game_screen.height()/200;	
			if(point.x>(ceil(247*game_screen.width()/320)) && point.y>(ceil(167*game_screen.height()/200)) && currentGame==1)
				return false;
			_mouseReel=point;
			Common::Point p;
			scaleMouse(p, point.x, point.y);
			clipMouse(p);

		
			if(currentGame==1){
				if(p.x>6)
					p.x-=6; // correction cursor bas droite du point d'appui gob1
				if(p.y>6)
					p.y-=6;
			}
			old=p;	
			if( currentGame==2 ){	
				if( gob3State == 0 || idLvlCourant == 550 || idLvlCourant == 680 || idLvlCourant == 690 || idLvlCourant == 600 || idLvlCourant == 710 || ((idLvlCourant==720 || idLvlCourant==560) && princeLastLeft==-1) )
						{
							
							if( p.x >= winkleLeft && p.x <= winkleLeft+22 && p.y >= winkleTop && p.y <= winkleTop+30){
									if( !winkleBusy )
										numGobliiin=1;
							}
							else if( p.x >= fingusLeft && p.x <= fingusLeft+22 && p.y >= fingusTop && p.y <= fingusTop+30 ){
									debugdot("**CheckInterface : fingusbusy= %d",fingusBusy); 									
									if( !fingusBusy )
										numGobliiin=3;
							}
						}else
						{
							
							if( p.x >= fingusLeft && p.x <= fingusLeft+22 && p.y >= fingusTop && p.y <= fingusTop+30 ){
							
									numGobliiin=3;
							}
							else if( p.x >= winkleLeft && p.x <= winkleLeft+22 && p.y >= winkleTop && p.y <= winkleTop+30 ){
							
									numGobliiin=1;//[gobliinButton setImage:[UIImage imageNamed:@"winkle_p.png"] forState:UIControlStateNormal];				
							}
							else if( p.x >= princeLeft && p.x <= princeLeft+22 && p.y >= princeTop && p.y <= princeTop+30 ){
							
									numGobliiin=2;//[gobliinButton setImage:[UIImage imageNamed:@"prince_f.png"] forState:UIControlStateNormal];	
							 }
						}				
				
			}
			if(currentGame==3){
				if( p.x >= firstCharactLeft && p.x <= firstCharactRight && p.y >= firstCharactTop && p.y <= firstCharactBottom )
									numberGobliin=1;
				else if( p.x >= secondCharactLeft && p.x <= secondCharactRight && p.y >= secondCharactTop && p.y <= secondCharactBottom )
									numberGobliin=2;

			}	
		return true;
		}	
	}
	
	return false;

}

void DefaultEventManager::check_interface(Common::Point p){
	if(bGameOverOn){
		debugdot("**DefaultEventManager : game Over touch ");	
		if(gameoverok.isClicked(p.x,p.y)){
				bSimulation=true;
				bSimulCode=false;
				bLvLOneLoaded=false;
				bRestart=true;
				idLvlCourant--;
			debugdot("**DefaultEventManager : touch ok : pos = %d,%d", p.x,p.y);				
		}
		if(gameovercancel.isClicked(p.x,p.y)){
			bSimulation =false;
			bExit=true;
			debugdot("**DefaultEventManager : touch cancel : pos = %d,%d", p.x,p.y);
		}
		return;
	}
	if(bGoHotspot){
		if(endlvlok.isClicked(p.x,p.y)){
			bSimulation=true;
			bSimulCode=false;
			bLvLOneLoaded=false;
			//bGoSim=true;
			cpt=0;
			
		}
		return;
	}
	/*if(hintView.isEnable()){
		hintView.check_interface(p);
		return;
	}*/
	if(saveLoadView.isEnable()){
		saveLoadView.check_interface(p);
		return;
	}
	
	if(menuButton.isClicked(p.x,p.y)){
			
			if(!bResume && !bMoveTo){
				if(currentGame==1)
					bResume=true;
				if(currentGame==2 && !fingusBusy && !winkleBusy && enable_pause){
				//	bFakeMousePause=true;
					bResume=true;
				}
				if(currentGame==3 && enable_pause && !charactereBusy ){
				//	bFakeMousePause=true;
					bResume=true;
				}
			}	
			
	}
	if(bPause){
		if( bExtras )
		{
			if(moveTo.isClicked(p.x,p.y)){
				if((!bMoveTo &&!fingusBusy && !winkleBusy && currentGame==2) ||(!bMoveTo && !charactereBusy && currentGame==3)){
					if(bBagOn)
						bResetActionMoveTo=true;
					bFakeMousePause=true;
					bResume=true;
					SimuMoveTo=true;
					fakePos=true;
					bNoUpdateHotspot=true;
					//bMoveTo=true;
					etatMoveTo=0;
				}
	
			}
			if(journal.isClicked(p.x,p.y)){
				if( !charactereBusy && currentGame==3){
					if(bBagOn)
						bResetActionMoveTo=true;
					bFakeMousePause=true;
					bResume=true;
					SimuJournal=true;
					fakePos=true;
					bNoUpdateHotspot=true;
					//bMoveTo=true;
					etatJournal=0;
				}
	
			}
			
			if(hint.isClicked(p.x,p.y)){
				JNI::showHintMenu(idLvlCourant);
			}
			if(back.isClicked(p.x,p.y)){
				debugdot("back from extras");
				bExtras = false;
			}
			return;
		}
		else
		{
			if(hint.isClicked(p.x,p.y) && currentGame==1){
				JNI::showHintMenu(idLvlCourant);
			}
			if(save.isClicked(p.x,p.y)){
				debugdot("saveClic");
				saveLoadView.enableDisplay(true);
				bSave=true;
			}
			if(extras.isClicked(p.x,p.y)){
				debugdot("extrasClic");
				bExtras = true;
			}
			if(settings.isClicked(p.x,p.y)){
				debugdot("settingsClic");
				JNI::showSettingsMenu();
			}
			if(load.isClicked(p.x,p.y)){
				debugdot("loadClic");
				saveLoadView.enableDisplay(true);
				bSave=false;
			}
			
			if(restart.isClicked(p.x,p.y)){
				if(currentGame==1){
					if(idLvlCourant!=1){
						debugdot("DefaultEventManager : checkInterface : clic sur restart");
						bPause=false;
						bExtras = false;
						bSimulation=true;
						bSimulCode=false;
						bLvLOneLoaded=false;
						posSimulation=Common::Point(121*game_screen.width()/320+game_screen.left, 113*game_screen.height()/200);
						bRestart=true;
						cptEsc = 8;
						idLvlToLoad=idLvlCourant;
						idLvlCourant--;
					} else {
						bPause=false;
						bExtras = false;
						bLoadingIntro=true;
						debugdot("**DefaultEventManager : reload lvl 1");
						bReload=true;
						bSimulation = false;
						g_engine->quitGame();
					}
				}
			}
			if(resume.isClicked(p.x,p.y)){
				debugdot("**DefaultEventManager : resume game");
				if(!bResume){
					if(currentGame!=1)
						bFakeMousePause=true;
					bResume=true;
				}
				
				
				//screen_contains(Common::Point(posx,posy),_mousePos);
			}
			if(leave.isClicked(p.x,p.y)){
				bPause=false;
				bExtras = false;
				debugdot("**DefaultEventManager : checkInterface : clic sur exit");
				bSimulation = false;
				bExit=true;
				
			}
			return;
		}
	}
	
	
	if(action.isClicked(p.x,p.y)){
		if(currentGame==1){
			if(!sendEvent){
				debugdot("sendEvent");					
				sendEvent=true;
			}
		}
		if(currentGame==2 || currentGame==3){
			if(!sendEventAction && !fingusBusy && !winkleBusy ){
				sendEventAction=true;
				etat=3;
			}
		}				
		
	}
	if(sendEventAction)
		return;
	
	if(onOff.isClicked(p.x,p.y)){	
			debugdot("Hotspot ON/OF !!!");			
			showHotspot=true;
		}else{
		
			if(switchgob.isClicked(p.x,p.y)){
				if(currentGame==1 && !nextGob && !fakePos)
					nextGob=true;
				if(currentGame==2 && !next){
							
						debugdot("**DefaultEventManager::fingusBusy= %d , winkleBusy=%d",fingusBusy,winkleBusy);
						debugdot("**DefaultEventManager::winkleLeft=%d , winkleTop=%d ,fingusLeft=%d , fingusTop=%d",winkleLeft, winkleTop,fingusLeft , fingusTop);
						//xPosCur = ((OSystem_IPHONE*)g_system)->_mouseX;
						//yPosCur = ((OSystem_IPHONE*)g_system)->_mouseY;
			
						//[self cursorCurrentGobliiin:((OSystem_IPHONE*)g_system)->_mouseX coordY:((OSystem_IPHONE*)g_system)->_mouseY];
			
						changeGobliin = true;
			
						float xTouch, yTouch;
						bool superpos = false;
			
						if( gob3State == 0 || idLvlCourant == 550 || idLvlCourant == 680 || idLvlCourant == 690 || idLvlCourant == 600 || idLvlCourant == 710 || ((idLvlCourant==720 || idLvlCourant==560) && princeLastLeft==-1) )
						{
							if( numberGobliin == 2 )
							{
								xTouch = fingusLeft + 11;
								yTouch = fingusTop + 15;
					
								if( xTouch >= winkleLeft && xTouch <= winkleLeft+22 && yTouch >= winkleTop && yTouch <= winkleTop+30 )
								{
									//NSLog(@"superposition A %d,%d",fingusLeft,winkleLeft);
									superpos = true;
									debugdot("Bouge ton gob");
									WarningMessage=true;
									//[self warningSuperposition];
								}
								else
								{
									if( !fingusBusy )
										numGobliiin=3;//[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
								}
							}
							else if( numberGobliin == 1 )
							{
								xTouch = winkleLeft + 11;
								yTouch = winkleTop + 15;	
					
								if( xTouch >= fingusLeft && xTouch <= fingusLeft+22 && yTouch >= fingusTop && yTouch <= fingusTop+30 )
								{
									//NSLog(@"superposition B %d,%d",fingusLeft,winkleLeft);
									superpos = true;
									debugdot("Bouge ton gob");
									WarningMessage=true;	
									//[self warningSuperposition];
								}
								else
								{
									if( !winkleBusy )
										numGobliiin=1;//[gobliinButton setImage:[UIImage imageNamed:@"winkle.png"] forState:UIControlStateNormal];
								}
							}
						}
						else
						{
							if( numberGobliin == 3 )
							{
								xTouch = fingusLeft + 11;
								yTouch = fingusTop + 15;
					
								if( xTouch >= princeLeft && xTouch <= princeLeft+22 && yTouch >= princeTop && yTouch <= princeTop+30 )
								{
									//NSLog(@"superposition A %d,%d",fingusLeft,princeLeft);
									superpos = true;
									debugdot("Bouge ton gob");
									WarningMessage=true;
									//[self warningSuperposition];
								}
								else
								{
									numGobliiin=3;//[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
								}
							}
							else if( numberGobliin == 1 )
							{
								xTouch = winkleLeft + 11;
								yTouch = winkleTop + 15;		
								debugdot("winkle =%d,%d", xTouch,yTouch);
								if( xTouch >= fingusLeft && xTouch <= fingusLeft+22 && yTouch >= fingusTop && yTouch <= fingusTop+30 )
								{
									//NSLog(@"superposition B %d,%d",fingusLeft,winkleLeft);
									superpos = true;
									debugdot("Bouge ton gob");
									WarningMessage=true;	
									//[self warningSuperposition];
								}
								else
								{
									numGobliiin=1;//[gobliinButton setImage:[UIImage imageNamed:@"winkle_p.png"] forState:UIControlStateNormal];
								}
							}	
							else if( numberGobliin == 2 )
							{
								xTouch = princeLeft + 11;
								yTouch = princeTop + 15;		
					
								if( xTouch >= winkleLeft && xTouch <= winkleLeft+22 && yTouch >= winkleTop && yTouch <= winkleTop+30 )
								{
									//NSLog(@"superposition C %d,%d",princeLeft,winkleLeft);
									superpos = true;
									debugdot("Bouge ton gob");
									WarningMessage=true;
									//[self warningSuperposition];
								}
								else
								{
									numGobliiin=2;//[gobliinButton setImage:[UIImage imageNamed:@"prince_f.png"] forState:UIControlStateNormal];
								}
							}	
						}
			
						if( !superpos )
						{
							if( yTouch <= 10 ) yTouch = 10;
							//g_system->warpMouse( xTouch , yTouch );	
				
							//cursorGobliiin.frame = CGRectMake(xTouch *480/320,yTouch *320/200, 40, 40);	
			
							_xTouch=xTouch;
							_yTouch=yTouch;
							fakePos=true;
							next=true;
							etat=2;
							debugdot("pos touch gob = %d,%d",posx,posy);
							/*[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
											[NSNumber numberWithFloat:xTouch], @"x",
											[NSNumber numberWithFloat:yTouch], @"y",nil]];	*/
						}
			
						//[NSTimer scheduledTimerWithTimeInterval:0.2 target:self selector:@selector(resetPosCursor) userInfo:nil repeats:NO];
					}
					if(currentGame==3 &&  !lockNext){
						debugdot("tu clic sur next!!!!!!!!!!!!!!!!!!!!");
						float xTouch, yTouch;
						bool superpos = false;
						if( numberGobliin == 1 && secondCharactEnable)
							{
								if(Top){
									xTouch = secondCharactLeft + 5;
									yTouch = secondCharactTop + 5;
								}else{
									xTouch = secondCharactRight -5;
									yTouch = secondCharactBottom - 5;
								}
								if( xTouch >= firstCharactLeft && xTouch <= firstCharactRight && yTouch >= firstCharactTop && yTouch <= firstCharactBottom )
								{
									//NSLog(@"superposition A %d,%d",fingusLeft,princeLeft);
									superpos = true;
									debugdot("Bouge ton gob");
									WarningMessage=true;
									//[self warningSuperposition];
								}
								else
								{
									numberGobliin=2;//[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
								}
							}
						else if( numberGobliin == 2 && firstCharactEnable)
							{
								if(Top){
									xTouch = firstCharactLeft + 5;
									yTouch = firstCharactTop + 5;
								}else{
									xTouch = firstCharactRight -5;
									yTouch = firstCharactBottom - 5;
								}	
								debugdot("winkle =%d,%d", xTouch,yTouch);
								if( xTouch >= secondCharactLeft && xTouch <= secondCharactRight && yTouch >= secondCharactTop && yTouch <= secondCharactBottom )
								{
									//NSLog(@"superposition B %d,%d",fingusLeft,winkleLeft);
									superpos = true;
									debugdot("Bouge ton gob");
									WarningMessage=true;	
									//[self warningSuperposition];
								}
								else
								{
									numberGobliin=1;//[gobliinButton setImage:[UIImage imageNamed:@"winkle_p.png"] forState:UIControlStateNormal];
								}
							}
						else superpos=true;
						if( !superpos )
						{
							if( yTouch <= 10 ) yTouch = 10;
							//g_system->warpMouse( xTouch , yTouch );	
				
							//cursorGobliiin.frame = CGRectMake(xTouch *480/320,yTouch *320/200, 40, 40);	
			
							_xTouch=xTouch;
							_yTouch=yTouch;
							fakePos=true;
							next=true;
							lockNext=true;
							etat=2;
							debugdot("pos touch gob = %d,%d",posx,posy);
							/*[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
											[NSNumber numberWithFloat:xTouch], @"x",
											[NSNumber numberWithFloat:yTouch], @"y",nil]];	*/
						}	

					}
				


				
			}else {
				if(skip.isClicked(p.x,p.y)){
					if(bIntro){
						bSkipIntro=true;
						bLoading=true;
						bIntro=false;
						bShowSkipButton=false;
					}else{
						bSkip=true;
						bShowSkipButton=false;
					}					
				}
			}			
		}
	
	
}


DefaultEventManager::DefaultEventManager(Common::EventSource *boss) :
	_buttonState(0),
	_modifierState(0),
	_shouldQuit(false),
	_shouldRTL(false),
	_mousePos(Common::Point(121,113)),
	_down(true),
	saveEtatFrom(-1),
	bLoopSimu(-1),
	
	bSlotOn(false),
	cptForce(0),
	bLoopMove(-1),
	bLoopSkip(-1),
	etatSkip(0),
	cptF(0),
	done(false),
	saveDTempo(-1),
	saveETempo(-1),
	saveTempo(-1),
	etatMoveTo(-1),
	etatJournal(-1),
	skipEvent(false),
	bLoopJournal(-1),
	bLoopInventory(-1),
	bActionLoop(-1),
	bLoopAction(-1),
	_confirmExitDialogActive(false) {
	debugdot("DefaultEventManager : game_screen : %d,%d",game_screen.width(),game_screen.height());
	SimuMoveTo=false;
	SimuJournal=false;
	enable_pause=false;
	sendEventAction=false;
	slotSave=-1;
	assert(boss);
	loadEtat=-1;
	saveEtat=-1;
	_mouseX=0;
	_mouseY=0;
	loadLoop=0;
	saveLoop=0;
	bResetAction=false;
	_dispatcher.registerSource(boss, false);
	_dispatcher.registerSource(&_artificialEventSource, false);
	bLoop2=-1;
	bLoop3=-1;	
	bLoop=-1;
	_dispatcher.registerObserver(this, kEventManPriority, false);
	
	bResetActionMoveTo=false;
	
	// Reset key repeat
	_currentKeyDown.keycode = 0;
	_currentKeyDown.ascii = 0;
	_currentKeyDown.flags = 0;

#ifdef ENABLE_VKEYBD
	_vk = new Common::VirtualKeyboard();
#endif
#ifdef ENABLE_KEYMAPPER
	_keymapper = new Common::Keymapper(this);
	// EventDispatcher will automatically free the keymapper
	_dispatcher.registerMapper(_keymapper);
	_remap = false;
#else
	_dispatcher.registerMapper(new Common::DefaultEventMapper());
#endif
}

DefaultEventManager::~DefaultEventManager() {
#ifdef ENABLE_VKEYBD
	delete _vk;
#endif
}

void DefaultEventManager::init() {
#ifdef ENABLE_VKEYBD
	if (ConfMan.hasKey("vkeybd_pack_name")) {
		_vk->loadKeyboardPack(ConfMan.get("vkeybd_pack_name"));
	} else {
		_vk->loadKeyboardPack("vkeybd_default");
	}
#endif
}

bool DefaultEventManager::pollEvent(Common::Event &event) {
	unsigned int delay;
	/*if(currentGame!=3)
		delay=45;
	else delay=50;*/
	if(bSkipCredits){
		bShowSkipButton=false;
		bSkipIntro=true;
		bSkipCredits=false;
		if(currentGame!=1)
			cptF=10;
		else cptF=3;
	}
	if(bLoopSkip!=-1 && bLoopSkip<3){
		bLoopSkip++;  debugdot("bLoopSkip = %d , bSkipIntro = %d",bLoopSkip,bSkipIntro); return false; }
	else if( bLoopSkip == 3 )
	{ etatSkip=0; bLoopSkip = -1; }


	if(bSkipIntro){
		switch (etatSkip){
			case 0: // escap down
				
				event.type = Common::EVENT_KEYDOWN;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_ESCAPE;
				event.kbd.ascii  = Common::ASCII_ESCAPE;
				debugdot("**Simulation : Skip esc down");
				etatSkip++;
					return true;break;
			case 1:	//escap up	
				cptF++;	
				debugdot("**Simulation :Skip esc up");
				event.type = Common::EVENT_KEYUP;
				event.kbd.flags = 0;
				event.kbd.keycode =  Common::KEYCODE_ESCAPE;
				event.kbd.ascii =  Common::ASCII_ESCAPE;
				
				if(currentGame!=1){
					if(cptF>=20){
						etatSkip=0;
						cptF=0;
						bSkipIntro=false;
						return true;
					}
				} else if(cptF>=6){
						etatSkip=0;
						cptF=0;
						bSkipIntro=false;
						return true;
					}
				bLoopSkip=0;
				return true;
				break;	
			default: etat=0	;return false;break;
		}	
	}
	if(bSkip){
		switch (etatSkip){
			case 0: // escap down
				
				event.type = Common::EVENT_KEYDOWN;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_ESCAPE;
				event.kbd.ascii  = Common::ASCII_ESCAPE;
				debugdot("**Simulation : Skip esc down");
				etatSkip++;
					return true;break;
			case 1:	//escap up	
				
				debugdot("**Simulation :Skip esc up");
				event.type = Common::EVENT_KEYUP;
				event.kbd.flags = 0;
				event.kbd.keycode =  Common::KEYCODE_ESCAPE;
				event.kbd.ascii =  Common::ASCII_ESCAPE;
				
				etatSkip=0;
			
				bSkip=false;
				return true;
			
				
				break;	
			default: etat=0	;return false;break;
		}	
	}


	//-----------------------
	//--------SIMULATION HACK GOB2
	//-----------------------
	if(currentGame==2 || currentGame==3){
		if((counter!=28 || bBagOn)  && bResume && !sendEventAction){
			if(bLoopInventory!=-1 && bLoopInventory<4){
				bLoopInventory++;  debugdot("bLoopInventory = %d ",bLoopInventory); return false; }
			else if( bLoopInventory == 4 )
			{  bLoopInventory = -1; }
			if(bResume){
				if(bPause){
					cpt++;
					if(cpt==1){
						debugdot("**RESUME : move on 100/150");
						_mousePos=Common::Point(100,160);
						event.type=Common::EVENT_MOUSEMOVE;
						return true;
				
					} else if (cpt>=7){
						//int i=0;
						//while(i<=100){ i++; debugdot("i= %d",i);}
						debugdot("**RESUME : move on prePos = %d,%d ",prePos.x,prePos.y);
						_mousePos=prePos;
						bPause=false;
						bExtras = false;
						bResume=false;
					
						bFakeMousePause=false;
						cpt=0;
						event.type=Common::EVENT_MOUSEMOVE;
						return true;
					}
				
				}else{	
					debugdot("!!!!!!!!!Pause!!!!! _mousePos = %d, %d ",_mousePos.x,_mousePos.y);
					bResume=false;
					done=false;
					bPause=true;
					bFakeMousePause=true;
					prePos=_mousePos;		
					_mousePos=Common::Point(1,0);	
					event.type=Common::EVENT_MOUSEMOVE;
					return true;			
				}
				
			
			}

		}else if (bResume && !done) { sendEventAction=true; etat=3; bFakeMousePause=true; done=true;}
			else if(bResume && done && !sendEventAction){ cptForce++; if(cptForce==10){ counter--; cptForce=0;}  debugdot("HEY T'ES BLOQUER !!!");}
	
	if(currentGame==2)
	{
		if( bActionLoop != -1 &&  bActionLoop< 12 )
		{ bActionLoop++;debugdot("**Simulation : ActionLoop = %d , cpt = %d",bActionLoop,cpt); return false; }
		else if( bActionLoop == 12 )
		{ cpt++; if(cpt==4) cpt=0;debugdot("**Simulation : cpt = %d",cpt); bActionLoop = -1; }

		if(bResetActionMoveTo){

			switch(cpt){
				case 0: event.type=Common::EVENT_RBUTTONDOWN;
					debugdot("**ResetAction :  clic right down");
					bActionLoop=7;
					return true;
					break;
				case 1: event.type=Common::EVENT_RBUTTONUP;
					debugdot("**ResetAction :  clic right up");
					bActionLoop=9;
				
					return true;
					break;
				case 2: event.type=Common::EVENT_RBUTTONDOWN;
					debugdot("**ResetAction :  clic right down");
					bActionLoop=7;
					return true;
					break;
				case 3: event.type=Common::EVENT_RBUTTONUP;
					debugdot("**ResetAction :  clic right up");
					bActionLoop=0;
					bResetActionMoveTo=false;
					cpt=0;
					return true;
					break;
				default : bResetActionMoveTo=false;cpt=0;bActionLoop=-1;return false;break;
			}	
		}
	}else {
		if( bActionLoop != -1 &&  bActionLoop< 12 )
		{ bActionLoop++;debugdot("**Simulation : ActionLoop2 = %d , cpt = %d",bActionLoop,cpt); return false; }
		else if( bActionLoop == 12 )
		{ cpt++; if(cpt==5) cpt=0;debugdot("**Simulation : cpt = %d",cpt); bActionLoop = -1; }

		if(bResetActionMoveTo){

			switch(cpt){
				
				case 0: event.type=Common::EVENT_RBUTTONDOWN;
					_mousePos=Common::Point(25,55);
					debugdot("**ResetAction :  clic left down");
					bActionLoop=5;
					return true;
					break;
				case 1: event.type=Common::EVENT_RBUTTONUP;
					debugdot("**ResetAction :  clic left up");
					bActionLoop=0;
					return true;
					break;
				case 2: event.type=Common::EVENT_LBUTTONDOWN;
					_mousePos=Common::Point(20,50);
					debugdot("**ResetAction :  clic right down");
					bActionLoop=5;
					return true;
					break;
				case 3: event.type=Common::EVENT_LBUTTONUP;
					debugdot("**ResetAction :  clic right up");
					bActionLoop=0;
					return true;
					break;
				case 4: event.type=Common::EVENT_MOUSEMOVE;
					debugdot("**ResetAction :  mouse move on 40,90");
					_mousePos=Common::Point(40,90);
					bActionLoop=0;
					bResetActionMoveTo=false;
					cpt=-1;
					return true;
					break;

				default:bResetAction=false;cpt=0;return false;break;
			}	
		}

	}

	if(bLoopMove!=-1 && bLoopMove<5){
	bLoopMove++;  debugdot("bLoopMove = %d , bMoveTo = %d",bLoopMove,bMoveTo); return false; }
	else if( bLoopMove == 5)
	{ etatMoveTo++; bLoopMove = -1; }


	if(SimuMoveTo){
		switch(etatMoveTo){
			case 0 : if(currentGame==2){
					debugdot("**MoveTo : move on 247/0");
					 _mousePos=Common::Point(247,0);
				}
				if(currentGame==3){
					debugdot("**MoveTo : move on 240/0");
					 _mousePos=Common::Point(240,0);
				}
				 event.type=Common::EVENT_MOUSEMOVE;
				 bLoopMove=0;
				 return true ;
				 break;
			case 1 : debugdot("**MoveTo : clic down 247/5");
				 _mousePos=Common::Point(240,5);
				 event.type=Common::EVENT_LBUTTONDOWN;
				 bMoveTo=true;
				 bNoUpdateHotspot=false;
				 bLoopMove=0;
				 return  true;
				 break;
			case 2 : debugdot("**MoveTo : clic up 247/0");
				 _mousePos=Common::Point(240,5);
				 event.type=Common::EVENT_LBUTTONUP;
				if(currentGame==3){
				//etatMoveTo++;
				 bLoopMove=0;
				}else etatMoveTo++;
				 return true;
				 break;
			case 3 : debugdot("**Moveto : move on oldPos = %d,%d",oldPos.x,oldPos.y);
				 _mousePos=oldPos;
				 event.type=Common::EVENT_MOUSEMOVE;
				debugdot("**Moveto : current Pos = %d,%d",_mousePos.x,_mousePos.y);
				 etatMoveTo=-1;
				 SimuMoveTo=false;
				 fakePos=false;
				 skipEvent=true;
				 return true;
				 break;
			default : etatMoveTo=-1; return false; break;
		}
	}

	if(currentGame==3){
		if(bLoopJournal!=-1 && bLoopJournal<5){
		bLoopJournal++;  debugdot("bLoopJournal = %d ",bLoopJournal); return false; }
		else if( bLoopJournal == 5)
		{ etatJournal++; bLoopJournal = -1; }


		if(SimuJournal){
			switch(etatJournal){
				case 0 :
				
					debugdot("**Journal : move on 126/0");
					_mousePos=Common::Point(126,0);
					
					 event.type=Common::EVENT_MOUSEMOVE;
					 bLoopJournal=0;
					 return true ;
					 break;
				case 1 : debugdot("**Journal : clic down 126/2");
					 _mousePos=Common::Point(126,2);
					 event.type=Common::EVENT_LBUTTONDOWN;
					 
					 bNoUpdateHotspot=false;
					 bLoopJournal=0;
					 return  true;
					 break;
				case 2 : debugdot("**Journal : clic up 126/2");
					 _mousePos=Common::Point(126,2);
					 event.type=Common::EVENT_LBUTTONUP;
					
					 bLoopJournal=0;
					
					 return true;
					 break;
				case 3 : debugdot("**Journal : move on oldPos = %d,%d",oldPos.x,oldPos.y);
					 _mousePos=oldPos;
					 event.type=Common::EVENT_MOUSEMOVE;
					debugdot("**Journal : current Pos = %d,%d",_mousePos.x,_mousePos.y);
					 etatJournal=-1;
					 SimuJournal=false;
					 fakePos=true;
					// skipEvent=true;
					 return true;
					 break;
				default : etatJournal=-1; return false; break;
			}
		}
	}
	

	
	
	if( bLoopAction != -1 &&  bLoopAction < 12 )
		{ bLoopAction++;debugdot("**Simulation : loopAction = %d , cpt = %d",bLoopAction,cpt); return false; }
		else if( bLoopAction == 12 )
		{ cpt++; if(cpt==7) cpt=0;debugdot("**Simulation : cpt = %d",cpt); bLoopAction = -1; }
	
	if(currentGame==2){
		if(bResetAction){		

			switch(cpt){
				case 0: _mousePos=Common::Point(160,0);
					event.type=Common::EVENT_MOUSEMOVE;
					debugdot("**ResetAction :  mouse move on 160,0");
					cpt++;
					return true;
					break;
				case 1: event.type=Common::EVENT_LBUTTONDOWN;
					debugdot("**ResetAction :  clic left down");
					bLoopAction=7;
					return true;
					break;
				case 2: event.type=Common::EVENT_LBUTTONUP;
					debugdot("**ResetAction :  clic left up");
					cpt++;
					return true;
					break;
				case 3: event.type=Common::EVENT_RBUTTONDOWN;
					debugdot("**ResetAction :  clic right down");
					bLoopAction=7;
					return true;
					break;
				case 4: event.type=Common::EVENT_RBUTTONUP;
					debugdot("**ResetAction :  clic right up");
					bLoopAction=7;
					return true;
					break;
				case 5: event.type=Common::EVENT_MOUSEMOVE;
					debugdot("**ResetAction :  mouse move on 130,100");
					_mousePos=Common::Point(130,100);
					bLoopAction=9;
					return true;
					break;
				case 6: event.type=Common::EVENT_MOUSEMOVE;
					debugdot("**ResetAction :  mouse move on 1,0");
					_mousePos=Common::Point(1,0);
					bLoopAction=0;
					bResetAction=false;
					return true;
					break;
				default:bResetAction=false;return false;break;
			}
		}
	} else {
		if(bResetAction){		

			switch(cpt){
				case 0: _mousePos=Common::Point(50,100);
					event.type=Common::EVENT_MOUSEMOVE;
					debugdot("**ResetAction :  mouse move on 50,100");
					bLoopAction=0;
					return true;
					break;
				case 1: event.type=Common::EVENT_RBUTTONDOWN;
					debugdot("**ResetAction :  clic left down");
					bLoopAction=7;
					return true;
					break;
				case 2: event.type=Common::EVENT_RBUTTONUP;
					debugdot("**ResetAction :  clic left up");
					bLoopAction=0;
					return true;
					break;
				case 3: event.type=Common::EVENT_LBUTTONDOWN;
					_mousePos=Common::Point(20,50);
					debugdot("**ResetAction :  clic right down");
					bLoopAction=7;
					return true;
					break;
				case 4: event.type=Common::EVENT_LBUTTONUP;
					debugdot("**ResetAction :  clic right up");
					bLoopAction=7;
					return true;
					break;
				case 5: event.type=Common::EVENT_MOUSEMOVE;
					debugdot("**ResetAction :  mouse move on 40,90");
					_mousePos=Common::Point(40,90);
					bLoopAction=9;
					return true;
					break;
				case 6: event.type=Common::EVENT_MOUSEMOVE;
					debugdot("**ResetAction :  mouse move on 1,0");
					_mousePos=Common::Point(1,0);
					bLoopAction=0;
					bResetAction=false;
					cpt=0;
					return true;
					break;
				default:bResetAction=false;cpt=0;return false;break;
			}
		}
	}

	
		
	if( bLoopSimu != -1 &&  bLoopSimu < 7 )
		{ bLoopSimu++;debugdot("bLoopSimu=%d",bLoopSimu); return false; }
		else if( bLoopSimu == 7 )
		{ etatSimulation++; bLoopSimu = -1; }
// !!!!!!!!!!!Begin hack LOADING
//debugdot("bWaitEnd =%d && StartSimu = %d", bWaitEnd,StartSimu);
		switch((InputSimulationGob23)etatSimulation)
			{
				case kEscDown1:
					if( cptEsc >= num && StartSimu)
					{
						return false;
					}
					event.type = Common::EVENT_KEYDOWN;
					event.kbd.flags = 0;
					event.kbd.keycode = Common::KEYCODE_ESCAPE;
					event.kbd.ascii  = Common::ASCII_ESCAPE;
					debugdot("**Simulation : esc down  here = %d",cptEsc);
					cptEsc++;
					etatSimulation++;
					return true;
				case kEscUp1:			
					if( cptEsc < num && StartSimu )
					{
						etatSimulation = -1;
						bLoopSimu = 0;
						
						debugdot("**Simulation : KescUp : cptEsc<8");
					}
					else if( cptEsc >= num && StartSimu )
					{
						debugdot("**Simulation : KescUp : cptEsc>=8");
						return false;
					}
					else if( cptEsc >= num && !StartSimu )
					{
						debugdot("**Simulation : KescUp : event Esc Up");
						etatSimulation++;
						cptEsc = 0;
					}
					//debugdot("**Simulation : esc up");
					event.type = Common::EVENT_KEYUP;
					event.kbd.flags = 0;
					event.kbd.keycode =  Common::KEYCODE_ESCAPE;
					event.kbd.ascii =  Common::ASCII_ESCAPE;
	
					return true;
				case kWaitIntro:
					debugdot("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!Wait Intro");
					debugdot("bWaitEnd =%d && StartSimu = %d && bStartWait = %d", bWaitEnd,StartSimu,bStartWait);	
					if(bWaitEnd && !StartSimu && currentGame==2){
						debugdot("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!wait end!!!!!!!!!!!!!!!!!!!!!!");
						bLoopSimu=0;
					}
					if(bWaitEnd && !StartSimu && bStartWait && currentGame==3){
						debugdot("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!wait end!!!!!!!!!!!!!!!!!!!!!!");
						bLoopSimu=0;
					}
					return false;
				case kMoveMouse:			
					event.type = Common::EVENT_MOUSEMOVE;
					_mousePos=Common::Point(1,0);
					
					debugdot("**Simulation : move : make Pause");
					bLoopSimu=0;
					return true;
				case kLaunchLoad:
					debugdot("**Simulation : launch load");
					loadEtat=0;
					_mouseY=0;_mouseX=20;
					slotSave=numSlot+1;	
					etatSimulation=-1;
					return false;
		}
	// !!!!!!!!!!!End hack LOADING
//WAIT FRAMES BETWEEN ACTIONS (SIMUL HACK)
	if( bLoop2 != -1 &&  bLoop2 < 5 )
	{ bLoop2++;debugdot("**Simulation : loop2 = %d , etat = %d",bLoop2,saveEtat); return false; }
	else if( bLoop2 == 5 )
	{ saveEtat++;debugdot("**Simulation : etat = %d",saveEtat); bLoop2 = -1; }
	//END


		if( saveTempo != -1 && saveTempo < 3 )
	{ saveTempo++; return false; }
	else if( saveTempo == 3 )
	{ saveTempo = -1; return true; }

	if( saveDTempo != -1 && saveDTempo < 10 )
	{ saveDTempo++;debugdot("saveDtemp = %d  save etat = %d",saveDTempo,saveEtat); return false; }
	else if( saveDTempo == 14 )
	{ saveDTempo = -1; return true; }
	
	if( saveETempo != -1 && saveETempo < 3 )
	{ saveETempo++;debugdot("saveEtemp = %d  save etat = %d",saveETempo,saveEtat); return false; }
	
	
	//SAVESIMULATION HACK
	if(currentGame==2){
		switch((saveSimulation)saveEtat)
		{
			case kLoadMove: 
					if(cpt==0){
						
						/*_mousePos=Common::Point(100,160);
						debugdot("**RESUME : move on 100/150");
						event.type=Common::EVENT_MOUSEMOVE;
						saveEtat=-1;*/
						cpt++;
						//bLoop2=0;
					}else if(cpt==1) {
						
						_mousePos=Common::Point(1,0);
						event.type=Common::EVENT_LBUTTONDOWN; 
						saveEtat=-1;
						cpt++;						
						bLoop2=0;
					}else {
						_mousePos=Common::Point(1,0);
						event.type=Common::EVENT_LBUTTONUP; 
											
						bLoop2=0;
					}
					
					return true; 
					break;
			case kSaveLD: //ldown
				if( saveLoop == 2 )
				{ _mouseY += slotSave * 10; }else _mouseY+=10*saveLoop;
				
				event.type = Common::EVENT_LBUTTONDOWN;
				event.mouse.x = _mouseX;
				event.mouse.y= _mouseY;
				//_mousePos = event.mouse;
				bLoop2 = 0;
				debugdot("clic left down in %d,%d",_mousePos.x,_mousePos.y);

				//saveEtat++;
				return true;		
				break;
			case kSaveLU: //lup	
				event.type = Common::EVENT_LBUTTONUP;
				event.mouse.x = _mouseX;
				event.mouse.y= _mouseY;
				_mousePos = event.mouse;
				debugdot("clic left up in %d,%d",_mousePos.x,_mousePos.y);
				/*if( saveLoop == 1 )
				{ _mouseY -= slotSave * 10; }*/
				saveEtatFrom = 2;
				saveLoop++;
				bLoop2=0;
				return true;
				break;
			case kSaveT: //tempo
				debugdot("SaveT  saveLoop= %d",saveLoop);
				if(saveLoop ==7) { saveLoop++; saveETempo=0; return false;}
				if( saveLoop == 8 )
				{ 	_mousePos=Common::Point(1,0);
					event.type=Common::EVENT_MOUSEMOVE;
					saveLoop = 0; saveEtat = -1; saveEtatFrom = -1;saveLoadView.enableDisplay(false); bFakeMousePause=false; bLoaded=true; _eventQueue.clear();return true; }
			
				if( saveLoop == 1 || saveLoop == 3 || saveLoop == 6 )
					saveDTempo = 0;
				else
					saveTempo = 0;
			
				if( saveEtatFrom == 5 )
				{ saveEtat = 6; return true; }
				else if( saveEtatFrom == 7 )
				{ saveEtat = 6; saveLoop = 7; return false; }
				else if( saveLoop == 4 )
				{ saveEtat = 4; saveLoop = 5; return false; }
				else 
				{ saveEtat = 1; return true; }
				saveEtatFrom = 3;
				return true;
				break;
			case kSaveAD: //adown	
				debugdot("A pressed down");
				event.type = Common::EVENT_KEYDOWN;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_a;
				event.kbd.ascii  = Common::ASCII_A;
			
				saveEtatFrom = 4;
				bLoop2=0;			
				
				return true;			
				break;
			case kSaveAU: //aup	
				debugdot("A pressed up");	
				event.type = Common::EVENT_KEYUP;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_a;
				event.kbd.ascii  = Common::ASCII_A;
			
				saveEtatFrom = 5;
				bLoop2=0;			
				saveEtat = 2;
				return true;			
				break;
			case kSaveED: //edown
				debugdot("enter pressed down");		
				event.type = Common::EVENT_KEYDOWN;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_RETURN;
				event.kbd.ascii  = Common::ASCII_RETURN;
			
				saveEtatFrom = 6;	
				bLoop2=0;
				return true;			
				break;
			case kSaveEU: //eup
				debugdot("enter pressed up");		
				event.type = Common::EVENT_KEYUP;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_RETURN;
				event.kbd.ascii  = Common::ASCII_RETURN;

				TimeDate td;
				struct tm tm;
				char* timeDateStr;
				debugdot("check time");
				g_system->getTimeAndDate(td);
				debugdot("have time");
				g_system->convertTimeDateToTm(td,tm);
				timeDateStr=asctime( &tm );
				timeDateStr[strlen(timeDateStr)-1]='\0';
				debugdot("timeDateStr= %s",timeDateStr);
				saveLoadView.Slot[numSlot].dateLastSave=(char*)malloc((strlen(timeDateStr)+1)*sizeof(char));
				strcpy(saveLoadView.Slot[numSlot].dateLastSave,timeDateStr);
				saveLoadView.Slot[numSlot].empty=false;
				g_system->writeSave2File();
				//drawString(game_screen.left+game_screen.width()*9/15,game_screen.height()*6/12,timeDateStr,50);
			
				saveEtatFrom = 7;	
				bLoop2=0;
				saveEtat = 2;
				if( saveLoop != 7 )
					saveLoop = 6;
				return true;		
				break;
		}
	}
	if(currentGame==3 ){
		switch((saveSimulation)saveEtat)
		{

			case kLoadMove: 
					if(cpt==0){
						
						/*_mousePos=Common::Point(100,160);
						debugdot("**RESUME : move on 100/150");
						event.type=Common::EVENT_MOUSEMOVE;
						saveEtat=-1;*/
						cpt++;
						//bLoop2=0;
					}else if(cpt==1) {
						
						_mousePos=Common::Point(1,0);
						event.type=Common::EVENT_LBUTTONDOWN; 
						saveEtat=-1;
						cpt++;						
						bLoop2=0;
					}else {
						_mousePos=Common::Point(1,0);
						event.type=Common::EVENT_LBUTTONUP; 
											
						bLoop2=0;
					}
					
					return true; 
					break;
			case kSaveLD: //ldown
				if( saveLoop == 2 )
				{ 
					_mouseX = 80;
					_mouseY = 35+ slotSave * 11; 
				}else _mouseY+=10*saveLoop;
				bSaving=true;
				event.type = Common::EVENT_LBUTTONDOWN;
				_mousePos.x = _mouseX;
				_mousePos.y = _mouseY;
				//_mousePos = event.mouse;
				bLoop2 = 0;
				debugdot("clic left down in %d,%d",_mousePos.x,_mousePos.y);

				//saveEtat++;
				return true;		
				break;
			case kSaveLU: //lup	
				event.type = Common::EVENT_LBUTTONUP;
				_mousePos.x = _mouseX;
				_mousePos.y = _mouseY;
				debugdot("clic left up in %d,%d",_mousePos.x,_mousePos.y);
				/*if( saveLoop == 1 )
				{ _mouseY -= slotSave * 10; }*/
				saveEtatFrom = 2;
				saveLoop++;
				bLoop2=0;
				return true;
				break;
			case kSaveT: //tempo
				debugdot("SaveT  saveLoop= %d",saveLoop);
				if( saveLoop == 5 ){
					saveDTempo = 0;
					saveLoop=6;
					return false;
				}
				if( saveLoop == 6 )
				{ 	_mousePos=Common::Point(1,0);
					event.type=Common::EVENT_MOUSEMOVE;
					saveLoop = 0; saveEtat = -1; saveEtatFrom = -1;saveLoadView.enableDisplay(false); bFakeMousePause=false; bLoaded=true;_eventQueue.clear(); return true; }
			
				if( saveLoop == 1 )
					saveDTempo = 0;
				else
					saveTempo = 0;
				if( saveEtatFrom == 7 )
				{ saveEtat = 6; saveLoop = 5; return false; }
				else if( saveLoop == 3 )
				{ saveEtat = 6; saveLoop = 4; return false; }
				else 
				{ saveEtat = 1; return true; }
				
				return true;
				break;
			case kSaveAD: //adown	
				
			case kSaveAU: //aup	
						
				break;
			case kSaveED: //edown
				debugdot("enter pressed down");		
				event.type = Common::EVENT_KEYDOWN;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_RETURN;
				event.kbd.ascii  = Common::ASCII_RETURN;
			
				saveEtatFrom = 6;	
				bLoop2=0;
				return true;			
				break;
			case kSaveEU: //eup
				debugdot("enter pressed up");		
				event.type = Common::EVENT_KEYUP;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_RETURN;
				event.kbd.ascii  = Common::ASCII_RETURN;

				TimeDate td;
				struct tm tm;
				char* timeDateStr;
				debugdot("check time");
				g_system->getTimeAndDate(td);
				debugdot("have time");
				g_system->convertTimeDateToTm(td,tm);
				timeDateStr=asctime( &tm );
				timeDateStr[strlen(timeDateStr)-1]='\0';
				debugdot("timeDateStr= %s:)",timeDateStr);
				saveLoadView.Slot[numSlot].dateLastSave=(char*)malloc((strlen(timeDateStr)+1)*sizeof(char));
				strcpy(saveLoadView.Slot[numSlot].dateLastSave,timeDateStr);
				saveLoadView.Slot[numSlot].empty=false;
				g_system->writeSave2File();
				//drawString(game_screen.left+game_screen.width()*9/15,game_screen.height()*6/12,timeDateStr,50);
			
				saveEtatFrom = 7;
				bLoop2=0;	
				saveEtat = 2;
				//saveLoop = 6;
				return true;		
				break;
		}
	}
	//END

	if( bLoop3 != -1 &&  bLoop3 < 5 )
		{ bLoop3++;debugdot("bLoop3=%d",bLoop3); return false; }
		else if( bLoop3 == 5 )
		{ loadEtat++; bLoop3 = -1; }
	
	//LOADSIMULATION HACK
	if(currentGame==2){
		switch((loadSimulation)loadEtat)
		{
			case kLoadMove: 
					if(cpt==0){
						
						/*_mousePos=Common::Point(100,160);
						debugdot("**load : move on 100/150");
						event.type=Common::EVENT_MOUSEMOVE;
						loadEtat=-1;*/
						cpt++;
						//bLoop3=0;
					}else if(cpt==1) {
						
						_mousePos=Common::Point(1,0);
						event.type=Common::EVENT_LBUTTONDOWN; 
						debugdot("**load : clic down on 1,0");
						loadEtat=-1;
						cpt++;						
						bLoop3=0;
					}else {
						_mousePos=Common::Point(1,0);
						event.type=Common::EVENT_LBUTTONUP; 
						debugdot("**load : clic up on 1,0");			
						bLoop3=0;
					}
					return true; 
					break;
			case kLoadLD: //ldown
						
				if( loadLoop == 3 )
				{ _mouseY += slotSave * 10; }
				
				event.type = Common::EVENT_LBUTTONDOWN;
				event.mouse.x = _mouseX;
				event.mouse.y= _mouseY;
				_mousePos = event.mouse;
				debugdot(" clic left down load %d,%d",_mouseX,_mouseY);
				bLoop3=0;
			
				return true;		
				break;
			case kLoadLU: //lup		
				event.type = Common::EVENT_LBUTTONUP;
				event.mouse.x = _mouseX;
				event.mouse.y= _mouseY;
				_mousePos = event.mouse;
				debugdot(" clic left  up load loadloop = %d !!! %d,%d",loadLoop,_mouseX,_mouseY);				
				if( loadLoop == 3 )
				{ _mouseY -= slotSave * 10; }
				loadLoop++;
				bLoop3=0;
				
			
				return true;
				break;
			case kLoadT: //tempo	
					
				debugdot(" kLoadT loadloop = %d !!! %d,%d",loadLoop,_mouseX,_mouseY);		
				if( loadLoop == 1 ){
					saveDTempo = 0;
					loadLoop++;
				}
				else
					saveTempo = 0;
			
				if( bSlotOn )
				{ _mouseY -= 10; bSlotOn = false; }
				if( loadLoop == 2 )
				{ loadEtat = 4; return false; }
				if( loadLoop == 4 ) {
				loadEtat=5; return false;}
				if( loadLoop == 5 )
				{ loadLoop = 0;bPause=false; cpt=0; bExtras = false; bFakeMousePause=false; loadEtat = -1;if(bSimu) bSimu=false;bLoaded=true; _eventQueue.clear(); return false; }
				else 
				{ loadEtat = 1; return true; }
				break;
			case kLoadSlot:			
							
				event.type = Common::EVENT_LBUTTONDOWN;
				_mouseY += 20; 	bSlotOn = true;
				event.mouse.x = _mouseX;
				event.mouse.y= _mouseY;
				_mousePos = event.mouse;
				
				debugdot(" clic left  down  slot load loadloop = %d !!! %d,%d",loadLoop,_mouseX,_mouseY);		
				bLoop3=0;
				loadEtat = 1;
				return true;
				break;
			case kLoadED: //edown
				debugdot("enter pressed down");		
				event.type = Common::EVENT_KEYDOWN;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_RETURN;
				event.kbd.ascii  = Common::ASCII_RETURN;
				bLoop3=0;
				
				
				return true;			
				break;
			case kLoadEU: //eup
				debugdot("enter pressed up");		
				event.type = Common::EVENT_KEYUP;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_RETURN;
				event.kbd.ascii  = Common::ASCII_RETURN;
				bLoop3=0;
				
				loadEtat = 2;
				loadLoop = 5;
				return true;		
				break;
		}
	}

	if(currentGame==3){
		switch((loadSimulation)loadEtat)
		{
			case kLoadMove: 
					debugdot("cpt=%d",cpt);
					if(cpt==0){
						
						/*_mousePos=Common::Point(100,160);
						debugdot("**RESUME : move on 100/150");
						event.type=Common::EVENT_MOUSEMOVE;
						saveEtat=-1;*/
						cpt++;
						//bLoop2=0;
					}else if(cpt==1) {
						
						_mousePos=Common::Point(1,0);
						event.type=Common::EVENT_LBUTTONDOWN; 
						loadEtat=-1;
						cpt++;						
						bLoop3=0;
					}else if(cpt==2){
						_mousePos=Common::Point(1,0);
						event.type=Common::EVENT_LBUTTONUP; 
						cpt++;					
						bLoop3=0;
					}else if(cpt==3){
		
						_mousePos=Common::Point(80, 35+ slotSave * 11 );
						event.type=Common::EVENT_MOUSEMOVE; 
						cpt++;
						loadEtat=-1;					
						bLoop3=0;
					}else if(cpt==4){
		
						_mousePos=Common::Point(80, 35+ (slotSave+1) * 11 );
						event.type=Common::EVENT_MOUSEMOVE; 
						cpt++;
						loadEtat=-1;					
						bLoop3=0;
					}else {

						_mousePos=Common::Point(80, 35+ slotSave * 11 );
						event.type=Common::EVENT_MOUSEMOVE; 
						cpt++;
										
						bLoop3=0;
					}
					return true; 
					break;
			case kLoadLD: //ldown
				if( loadLoop == 2 )
				{ 	
					_mouseX = 80;
					_mouseY = 35 + slotSave * 11;
				
				}else _mouseY+=25*loadLoop;
				bLoading=true;
				event.type = Common::EVENT_LBUTTONDOWN;
				_mousePos.x = _mouseX;
				_mousePos.y= _mouseY;
				debugdot(" clic left down load %d,%d",_mouseX,_mouseY);
				bLoop3=0;
			
				return true;		
				break;
			case kLoadLU: //lup		
				event.type = Common::EVENT_LBUTTONUP;
				_mousePos.x = _mouseX;
				_mousePos.y= _mouseY;
				debugdot(" clic left  up load %d,%d",_mouseX,_mouseY);				
				
				loadLoop++;
				bLoop3=0;
			
				return true;
				break;
			case kLoadT: //tempo	
					debugdot("kLoadT");		
				
				
				if( loadLoop == 5 )
				{ 	_mousePos=Common::Point(1,0);
					event.type=Common::EVENT_MOUSEMOVE;
					loadLoop = 0; loadEtat = -1; bPause=false; bExtras= false; bLoading=true; bFakeMousePause=false; _eventQueue.clear(); bLoaded=true;saveLoadView.enableDisplay(false);if(bSimu) bSimu=false; return true; }
			
				if( loadLoop == 1 )
					saveDTempo = 0;
				else
					saveTempo = 0;
				
				if( loadLoop == 3 )
				{ loadEtat = 5; loadLoop = 4; return false; }
				else if(loadLoop==2){
					loadEtat=0; return false;}
				else{ loadEtat = 1; return true; }
				
				return true;
				break;			
				
			case kLoadSlot:			
							
				break;
			case kLoadED: //edown
				debugdot("enter pressed down");		
				event.type = Common::EVENT_KEYDOWN;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_RETURN;
				event.kbd.ascii  = Common::ASCII_RETURN;
			
				
				bLoop3=0;
				return true;			
				break;
			case kLoadEU: //eup
				debugdot("enter pressed up");		
				event.type = Common::EVENT_KEYUP;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_RETURN;
				event.kbd.ascii  = Common::ASCII_RETURN;
			
				bLoop3=0;
				loadEtat = 2;
				loadLoop = 5;
				return true;		
				break;
		}
	}
	//END


	//WAIT FRAMES BETWEEN ACTIONS (SIMUL HACK)
		if( bLoop != -1 &&  bLoop < 7 )
		{ bLoop++;fakePos=true;debugdot("bLoop=%d",bLoop); return false; }
		else if( bLoop == 7 )
		{ etat++; bLoop = -1; }
		//END
	
		
		//debugdot("**Simulation : etat = %d", etat);
		//SIMULATION HACK
		//debugdot("counter=%d, bBagOn=%d , sendevent action=%d, next =%d",counter,bBagOn,sendEventAction, next);
		if((counter!=28 || bBagOn)  && next && !sendEventAction){
			if(bLoopInventory!=-1 && bLoopInventory<4){
				bLoopInventory++;  debugdot("bLoopInventory = %d ",bLoopInventory); return false; }
			else if( bLoopInventory == 4 )
			{  bLoopInventory = -1; }
	
			if(next){
				switch((InputSimulation)etat)
				{
					case kMouseMove:
						if(cpt>=1){
							_xTouch=oldPos.x;
							_yTouch=oldPos.y;
						
						}
						event.type = Common::EVENT_MOUSEMOVE;	
						_mousePos=Common::Point(_xTouch,_yTouch);	
						event.mouse=_mousePos;
						posx=(_xTouch)*game_screen.width()/320+game_screen.left;posx= (_yTouch)*game_screen.height()/200;
						debugdot("**Simulation : mouse pos Switch : %d,%d  pos reel = %d,%d", event.mouse.x,event.mouse.y,_mousePos.x,_mousePos.y);
						debugdot("**Simulation : mouse move Switch");
						if(cpt<1){
							debugdot("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!first!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
							etat++;
							cpt++;
							fakePos=true;
						
						}
						else {
							etat=-1;
							cpt=0;
							next=false;
							debugdot("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!END!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
						
							fakePos=true;
							//fakePos=false;
						}
					
						return true;
					case kClickDown:			
						event.type = Common::EVENT_LBUTTONDOWN;
						_mousePos=Common::Point(_xTouch,_yTouch);	
						event.mouse=_mousePos;
						posx=(_xTouch)*game_screen.width()/320+game_screen.left;posx= (_yTouch)*game_screen.height()/200;
						debugdot("**Simulation : click down Switch");
						bLoop = 0;
						fakePos=true;
						return true;
					case kClickUp:
						event.type = Common::EVENT_LBUTTONUP;
						_mousePos=Common::Point(_xTouch,_yTouch);	
						event.mouse=_mousePos;
						posx=(_xTouch)*game_screen.width()/320+game_screen.left;posx= (_yTouch)*game_screen.height()/200;
						debugdot("**Simulation : clic up Switch");
						etat=2;
						fakePos=true;
						//bLoop=0;	
						//next=false;
						done=false;
						return true;
					default : etat=-1; break;
				
				}
				
			}
		}else if (next && !done) { debugdot("SendAvction== truuuuueee !!!!!!!!!");sendEventAction=true; etat=3; fakePos=true; done=true;}
			else if(next && done && !sendEventAction){ cptForce++; if(cptForce==10){ counter--; cptForce=0;} debugdot("HEY T'ES BLOQUER !!!");}
			


		if(sendEventAction){
			switch((InputSimulation)etat)
			{
				case kClickDown:			
					event.type = Common::EVENT_RBUTTONDOWN;
					/*event.mouse = _mousePos;
					screen_contains(event.mouse,_mousePos);
					posx=_mouseReel.x,posx=_mouseReel.y;
					fakePos=true;*/
					if(_mousePos.x>217 && currentGame==2)
						_mousePos.x=217;
					debugdot("**Simulation : click down NextAction counter = %d",counter);
					/*if(counter!=28)
						bNoUpdateHotspot=true;
					else bNoUpdateHotspot=false;*/
					
					bLoop = 0;
					return true;
				case kClickUp:
					/*if(_mousePos.x>181)
						_mousePos.x=181;*/
					event.type = Common::EVENT_RBUTTONUP;
					/*event.mouse = Common::Point((_xTouch)*game_screen.width()/320+game_screen.left, (_yTouch)*game_screen.height()/200+1);
					screen_contains(event.mouse,_mousePos);
					posx=_mouseReel.x,posx=_mouseReel.y;*/
					debugdot("**Simulation : clic up NextAction");
					etat=-1;
					//bBagOn=!bBagOn;
					sendEventAction=false;
					//bNoUpdateHotspot=true;
					if(bResume || next){
						bLoopInventory=0;
						etat=2;
					}
					return true;
				default : etat=-1; break;
				
			}

		}

	}
			
	
	

	//-----------------------
	//--------SIMULATION HACK GOB1
	//-----------------------
	
	if(currentGame==1){

	if(bResume){
		if(_down){
			debugdot("**RESUME : Escape down");
			event.type = Common::EVENT_KEYDOWN;

			
			event.kbd.flags=0;
			_down=false;
			return true;
		}else{
			cpt++;
			if(cpt>=10){
				debugdot("**RESUME : Escape up");
				event.type = Common::EVENT_KEYUP;
				event.kbd.flags=0;
				_down=true;
				bResume=false;
				bPause=!bPause;	
				cpt=0;
			}
			
		}
		event.kbd.keycode = Common::KEYCODE_ESCAPE;
		event.kbd.ascii = Common::ASCII_ESCAPE;
		return true;
	}

	if((sendEvent) || (kTempo!=15))
		{	
		
			//debugdot("sendEvent = %d",sendEvent);
			//debugdot("kTempo = %d",kTempo);
			//pushEvent(3,posx,posy,0,0,0);
			
			
			kTempo--;
			if(kTempo==10){		
				event.type = Common::EVENT_RBUTTONDOWN;	
				return true;
			}
				
			if(kTempo==5){
				event.type = Common::EVENT_RBUTTONUP;
				return true;
			}
				
			if(kTempo==3){
				//pushEvent(5,posx,posy,0.65,0,0);
				kTempo=15;
				sendEvent=false;
			}
	}
		
	if((nextGob)|| (kTempoSwitch!=12)){
			
		fakePos=true;
		
		//debugdot("nextGob = %d",nextGob);
		//debugdot("kTempoSwitch = %d",kTempoSwitch);
		
		kTempoSwitch--;
			
		if(kTempoSwitch==11){
			debugdot("clic mouse move");
			event.type = Common::EVENT_MOUSEMOVE;
			_mousePos = Common::Point(167,185);
			return true;
		}
		if(kTempoSwitch==10){
			debugdot("clic left down");
			event.type = Common::EVENT_LBUTTONDOWN;
			_mousePos =Common::Point(167,185);
			return true;
		}						
		if(kTempoSwitch==5){
			debugdot("clic left up");
			event.type = Common::EVENT_LBUTTONUP;
			_mousePos =Common::Point(167,185);
			return true;		
		}
		if(kTempoSwitch==3){
			debugdot("clic mouse move");
			kTempoSwitch=12;
			nextGob=false;
			event.type = Common::EVENT_MOUSEMOVE;
			_mousePos =oldPos;
			return true;
		}
	}
	
	

	

//PRE SIMULATION HACK
		preSimulation();
	
//WAIT FRAMES BETWEEN ACTIONS (SIMUL HACK)
		if( bLoop != -1 &&  bLoop < 7 )
		{ bLoop++; return false; }
		else if( bLoop == 7 )
		{ etat++; bLoop = -1; }
		//END
	
		//WAIT FRAMES BETWEEN ACTIONS (SIMUL HACK)
		if( bLoop2 != -1 &&  bLoop2 < 12 )
		{ bLoop2++;debugdot("**Simulation : loop2 = %d , etat = %d",bLoop2,etat); return false; }
		else if( bLoop2 == 12 )
		{ etat++;debugdot("**Simulation : etat = %d",etat); bLoop2 = -1; }
		//END
		//debugdot("**Simulation : etat = %d", etat);
		//SIMULATION HACK
	
	

		//HACK GO
	if( idLvlCourant == 8 )
	{
		if( bGo != -1 && bGo < 25 )
		{ bGo++; return false; }
		else if( bGo == 25)
		{ bGo = -1;return true; }
	}
	else
	{
		if( bGo != -1 && bGo < 17 )
		{ bGo++; return false; }
		else if( bGo == 17 )
		{ bGo = -1; return true; }	
	}
	//END

		if(bSimulGO){
			switch((InputSimulation)etat)
			{
				case kMouseMove:
					
					event.type = Common::EVENT_MOUSEMOVE;			
					event.mouse = Common::Point((myHackSpotL+5)*game_screen.width()/320+game_screen.left, (myHackSpotT+5)*game_screen.height()/200);
					screen_contains(event.mouse,_mousePos);
					posx=_mouseReel.x,posx=_mouseReel.y;
					debugdot("**Simulation : mouse pos GO : %d,%d  pos reel = %d,%d", event.mouse.x,event.mouse.y,_mousePos.x,_mousePos.y);
					debugdot("**Simulation : mouse move GO");
					etat++;
					return true;
				case kClickDown:			
					event.type = Common::EVENT_LBUTTONDOWN;
					event.mouse = Common::Point((myHackSpotL+5)*game_screen.width()/320+game_screen.left, (myHackSpotT+5)*game_screen.height()/200);				bGoHotspot=false;
					screen_contains(event.mouse,_mousePos);
					posx=_mouseReel.x,posx=_mouseReel.y;
					debugdot("**Simulation : click down GO");
					bLoop = 0;
					return true;
				case kClickUp:
					event.type = Common::EVENT_LBUTTONUP;
					event.mouse = Common::Point((myHackSpotL+5)*game_screen.width()/320+game_screen.left, (myHackSpotT+5)*game_screen.height()/200);
					screen_contains(event.mouse,_mousePos);
					posx=_mouseReel.x,posx=_mouseReel.y;
					debugdot("**Simulation : clic up GO");
					

					/*if(cpt<1){
						bGo=0;
						etat=2;
						cpt++;
					}else{*/
						cpt=0;
						etat=-1;
						bGoHotspot=false;
					//}
					return true;
				default : etat=-1; break;
				
			}
		}else if(!bSimulGOver) {
			
			switch((InputSimulation)etat)
			{
				case kEscDown:
					if( cptEsc >= 8 && !bSimulCode )
					{
						return false;
					}
					event.type = Common::EVENT_KEYDOWN;
					event.kbd.flags = 0;
					event.kbd.keycode = Common::KEYCODE_ESCAPE;
					event.kbd.ascii  = Common::ASCII_ESCAPE;
					debugdot("**Simulation : esc down");
					etat++;
					return true;
				case kEscUp:			
					if( cptEsc < 8 && !bSimulCode )
					{
						etat = -1;
						bLoop = 0;
						cptEsc++;
						debugdot("**Simulation : KescUp : cptEsc<8");
					}
					else if( cptEsc >= 8 && !bSimulCode )
					{
						debugdot("**Simulation : KescUp : cptEsc>=8");
						return false;
					}
					else if( cptEsc >= 8 && bSimulCode )
					{
						debugdot("**Simulation : KescUp : event Esc Up");
						etat++;
						cptEsc = 0;
					}
					//debugdot("**Simulation : esc up");
					event.type = Common::EVENT_KEYUP;
					event.kbd.flags = 0;
					event.kbd.keycode =  Common::KEYCODE_ESCAPE;
					event.kbd.ascii =  Common::ASCII_ESCAPE;
	
					return true;
				case kMouseMove:	
					event.type = Common::EVENT_MOUSEMOVE;			
					event.mouse = Common::Point(121*game_screen.width()/320+game_screen.left, 113*game_screen.height()/200);
					screen_contains(event.mouse,_mousePos);
					posx=_mouseReel.x,posx=_mouseReel.y;
					debugdot("**Simulation : mouse pos : %d,%d ", event.mouse.x,event.mouse.y);
					debugdot("**Simulation : mouse move");
					etat++;
					return true;
				case kClickDown:			
					event.type = Common::EVENT_LBUTTONDOWN;
					event.mouse = Common::Point(121*game_screen.width()/320+game_screen.left, 113*game_screen.height()/200);
					screen_contains(event.mouse,_mousePos);
					posx=_mouseReel.x,posx=_mouseReel.y;
					debugdot("**Simulation : click down");
					bLoop = 0;
					return true;
				case kClickUp:
					event.type = Common::EVENT_LBUTTONUP;
					event.mouse = Common::Point(121*game_screen.width()/320+game_screen.left, 113*game_screen.height()/200);
					screen_contains(event.mouse,_mousePos);
					posx=_mouseReel.x,posx=_mouseReel.y;
					debugdot("**Simulation : clic up");
					bLoop2 = 0;	
					return true;
				case kKeyDown:
					event.type = Common::EVENT_KEYDOWN;
					event.kbd.flags = 0;
					event.kbd.keycode = m_codeLvLKeyCode[idLvlToLoad-2][cpt];
					event.kbd.ascii  = m_codeLvLAscii[idLvlToLoad-2][cpt];
					debugdot("**Simulation : keydown : cpt = %d",cpt);
					etat++;
					return true;
				case kKeyUp:				
					event.type = Common::EVENT_KEYUP;
					event.kbd.flags = 0;
					event.kbd.keycode = m_codeLvLKeyCode[idLvlToLoad-2][cpt];
					event.kbd.ascii  = m_codeLvLAscii[idLvlToLoad-2][cpt];
					debugdot("**Simulation : keyup : cpt = %d",cpt);
					if( cpt < 6 )
					{ etat--; cpt++; }
					else 
						etat++;
					return true;
				case kEnterDown:			
					event.type = Common::EVENT_KEYDOWN;
					event.kbd.flags = 0;
					event.kbd.keycode = Common::KEYCODE_RETURN;
					event.kbd.ascii  = Common::ASCII_RETURN;
					debugdot("**Simulation : enter down");
					etat++;
				
					return true;
				case kEnterUp:
					event.type = Common::EVENT_KEYUP;
					event.kbd.flags = 0;
					event.kbd.keycode = Common::KEYCODE_RETURN;
					event.kbd.ascii  = Common::ASCII_RETURN;
					debugdot("**Simulation : enter up");
					etat = -1;
					cpt = 0;
					bRestart=false;
			
					return true;
			}
			}else {
			//debugdot("**Simulation : Game over !!");
				switch((InputSimulation)etat)
				{
					case kMouseMove:
					
						event.type = Common::EVENT_MOUSEMOVE;			
						_mousePos = Common::Point(162, 58);
						
						debugdot("**Simulation GAME OVER : mouse pos : %d,%d  pos reel = %d,%d", event.mouse.x,event.mouse.y,posx,posy);
						debugdot("**Simulation GAME OVER : mouse move");
						etat++;
						return true;
					case kClickDown:			
						event.type = Common::EVENT_LBUTTONDOWN;
						
						
						debugdot("**Simulation GAME OVER : click down");
						bLoop = 0;
						return true;
					case kClickUp:
						event.type = Common::EVENT_LBUTTONUP;
						
						bSimulation=false;
						bLoading=true;
						etat=-2;
						bLoop2= 0;
						idLvlCourant++;
						debugdot("**Simulation GAME OVER : clic up  idlvlcourant = %d",idLvlCourant);
						return true;
					default : etat=-1; break;
				
				}
			//END
			}
		
			
	}
	
	uint32 time = g_system->getMillis();
	bool result = false;

	_dispatcher.dispatch();
	//debugdot("_eventQueue.empty()=%d",_eventQueue.empty());
	if (!_eventQueue.empty()) {
		event = _eventQueue.pop();
		
		debugdot("**DefaultEvent : SkipEvent⁼ %d",skipEvent);
		if(skipEvent){
			skipEvent=false;
			result=false;
		}else result = true;
	}
	//debugdot("DefaultEventManager :: result = %d",result);
	if (result) {
		event.synthetic = false;
		switch (event.type) {
		case Common::EVENT_KEYDOWN:
			if(bGoHotspot) return false;
			if(event.kbd.keycode == Common::KEYCODE_ESCAPE){
				if( bInGame && !bSimulation){
					/*if(hintView.isEnable()){
							hintView.enableDisplay(false);
							return false;
						}*/
					if(currentGame==1 && !bMoveTo){
						debugdot("!!!!!!!!!Pause");
						bResume=true;
						return false;
					}
				
				
					if(currentGame==2){
						
						if(!saveLoadView.isEnable() ){
							if(!fingusBusy && !winkleBusy && enable_pause && !bMoveTo){
								//bFakeMousePause=true;
								bResume=true;
							}
						}else
							saveLoadView.enableDisplay(false);
						return false;
					}
					if(currentGame==3){
						
						if(!saveLoadView.isEnable() ){
							if(enable_pause && !charactereBusy && !bMoveTo ){
								//bFakeMousePause=true;
								bResume=true;
							}
						}else
							saveLoadView.enableDisplay(false);
						return false;
					}
				}
				if(bShowSkipButton){
					if(bIntro){
						debugdot("bSkipIntro");
						bSkipIntro=true;
						bLoading=true;
						bIntro=false;
						bShowSkipButton=false;
					}
					else{
						bSkip=true;
						bShowSkipButton=false;
					}				
				}
				if(bIntro)
					return false;
				
				
				//screen_contains(Common::Point(posx,posy),_mousePos);
			}// else {
				debugdot("**DefaultManagerEvent : not in pause");
				_modifierState = event.kbd.flags;
				// init continuous event stream
				_currentKeyDown.ascii = event.kbd.ascii;
				_currentKeyDown.keycode = event.kbd.keycode;
				_currentKeyDown.flags = event.kbd.flags;
				_keyRepeatTime = time + kKeyRepeatInitialDelay;
		
				if (event.kbd.keycode == Common::KEYCODE_BACKSPACE) {
					// WORKAROUND: Some engines incorrectly attempt to use the
					// ascii value instead of the keycode to detect the backspace
					// key (a non-portable behavior). This fails at least on
					// Mac OS X, possibly also on other systems.
					// As a workaround, we force the ascii value for backspace
					// key pressed. A better fix would be for engines to stop
					// making invalid assumptions about ascii values.
					event.kbd.ascii = Common::KEYCODE_BACKSPACE;
					_currentKeyDown.ascii = Common::KEYCODE_BACKSPACE;
				}
			//}
				break;
		

		case Common::EVENT_KEYUP:
			if(bGoHotspot) return false;
			if(event.kbd.keycode == Common::KEYCODE_ESCAPE && bInGame && !bSimulation){
				
				if(currentGame==2 || bIntro)
					return false;
			}
			_modifierState = event.kbd.flags;
				if (event.kbd.keycode == _currentKeyDown.keycode) {
					// Only stop firing events if it's the current key
					_currentKeyDown.keycode = 0;
				}
			
			break;

		case Common::EVENT_MOUSEMOVE:
			debugdot("**Default-events : pollEvent : event.mouse = %d, %d ",event.mouse.x,event.mouse.y);
			if(!screen_contains(event.mouse,_mousePos)){
				
				return false;
			}
			
				
			debugdot("**Default-events : pollEvent : _mousePosX = %d, _mousePosY = %d ",_mousePos.x,_mousePos.y);
			posx=_mouseReel.x;posy=_mouseReel.y;
			break;

		case Common::EVENT_LBUTTONDOWN:
			debugdot("DefaultEventManager::pollEvent  : Reel mouse pos x,y = %d,%d",event.mouse.x,event.mouse.y);
			if(!screen_contains(event.mouse,_mousePos)){
				debugdot("Not in the screen");
				return false;
			}

#ifndef TIMER_HOTSPOT
	//debugdot("if n def  TIMER _ hotspot");
	showHotspot=false;
#endif
			
			bNoUpdateHotspot=false;
			//bBagOn=false;
			posx=_mouseReel.x;posy=_mouseReel.y;

		debugdot("DefaultEventManager::pollEvent : pos mouse x,y %d,%d",_mousePos.x,_mousePos.y);
			
			_buttonState |= LBUTTON;
			break;

		case Common::EVENT_LBUTTONUP:
			debugdot("DefaultEventManager::pollEvent : pos mouse x,y %d,%d",_mousePos.x,_mousePos.y);
			if(!screen_contains(event.mouse,_mousePos)){
				check_interface(event.mouse);
				return false;
			}
			
			if(bMoveTo/* && currentGame==2*/)
				bMoveTo=false;
			bNoUpdateHotspot=false;
			//bBagOn=false;
			posx=_mouseReel.x;posy=_mouseReel.y;
				
			_buttonState &= ~LBUTTON;
			break;

		case Common::EVENT_RBUTTONDOWN:
			if(!screen_contains(event.mouse,_mousePos)){
				
				return false;
			}
			_buttonState |= RBUTTON;
			break;

		case Common::EVENT_RBUTTONUP:
debugdot("DefaultEventManager::pollEvent : pos mouse x,y %d,%d",event.mouse.x,event.mouse.y);
debugdot("DefaultEventManager::pollEvent : rButtonUp");
			if(!screen_contains(event.mouse,_mousePos)){
				
				return false;
			}
			_buttonState &= ~RBUTTON;
			break;

		case Common::EVENT_MAINMENU:
/*debugdot("DefaultEventManager::pollEvent:: MainMenu");
			if (g_engine && !g_engine->isPaused())
				g_engine->openMainMenuDialog();

			if (_shouldQuit)
				event.type = Common::EVENT_QUIT;
			else if (_shouldRTL)
				event.type = Common::EVENT_RTL;
			break;*/
#ifdef ENABLE_VKEYBD
		case Common::EVENT_VIRTUAL_KEYBOARD:
			debugdot("EventVirtual keyboard");
			if (_vk->isDisplaying()) {
				_vk->close(true);
			} else {
				if (g_engine)
					g_engine->pauseEngine(true);
				_vk->show();
				if (g_engine)
					g_engine->pauseEngine(false);
				result = false;
			}
			break;
#endif
#ifdef ENABLE_KEYMAPPER
		case Common::EVENT_KEYMAPPER_REMAP:
			if (!_remap) {
				_remap = true;
				Common::RemapDialog _remapDialog;
				if (g_engine)
					g_engine->pauseEngine(true);
				_remapDialog.runModal();
				if (g_engine)
					g_engine->pauseEngine(false);
				_remap = false;
			}
			break;
#endif
		case Common::EVENT_RTL:
			if (ConfMan.getBool("confirm_exit")) {
				if (g_engine)
					g_engine->pauseEngine(true);
				GUI::MessageDialog alert(_("Do you really want to return to the Launcher?"), _("Launcher"), _("Cancel"));
				result = _shouldRTL = (alert.runModal() == GUI::kMessageOK);
				if (g_engine)
					g_engine->pauseEngine(false);
			} else
				_shouldRTL = true;
			break;

		case Common::EVENT_MUTE:
			if (g_engine)
				g_engine->flipMute();
			break;

		case Common::EVENT_QUIT:
			debugdot("**DefaultEventmanager : shouldQuit");
			if (ConfMan.getBool("confirm_exit")) {
				if (_confirmExitDialogActive) {
					result = false;
					break;
				}
				_confirmExitDialogActive = true;
				if (g_engine)
					g_engine->pauseEngine(true);
				GUI::MessageDialog alert(_("Do you really want to quit?"), _("Quit"), _("Cancel"));
				result = _shouldQuit = (alert.runModal() == GUI::kMessageOK);
				if (g_engine)
					g_engine->pauseEngine(false);
				_confirmExitDialogActive = false;
			} else
				_shouldQuit = true;

			break;

		default:
			break;
		}
	} else {
		// Check if event should be sent again (keydown)

		if (_currentKeyDown.keycode != 0 && _keyRepeatTime < time) {
			debugdot("**DefaultManagerEvent : fire event");
			// fire event
			event.type = Common::EVENT_KEYDOWN;
			event.synthetic = true;
			event.kbd.ascii = _currentKeyDown.ascii;
			event.kbd.keycode = (Common::KeyCode)_currentKeyDown.keycode;
			event.kbd.flags = _currentKeyDown.flags;
			_keyRepeatTime = time + kKeyRepeatSustainDelay;
			result = true;
		}
	}

	return result;
}

bool DefaultEventManager::preSimulation(){
	//WAIT FRAMES TO LAUNCH SIMUL HACK
	//debugdot("**Simulation : enter presimul");
	if( (strcmp( niveau, "AVT003.AVJ" )  == 0 || bRestart || /*bGoSim*/ bGoHotspot) && (!bSimulCode) && bSimulation )
	{

		if( bCptSimul == 10 )
		{
			debugdot("**Simulation : bCptSimul = 10");
			if(bGameOverOn){
				bGameOverOn=false;
				bSimulGOver=true;
				bSimulGO=false;
				etat=2;
			}
			else {
				if(bGoHotspot){
					etat=2;
					bSimulGO=true;
				  //    	bGoSim=false;
				//	bGoHotspot=false;
					bLoop=0;
				}
				else{ 
					bSimulGO=false;
					bSimulGOver=false;
					if (bRestart)
						etat=2;
					else
						etat=0;
				}
			}
			bSimulCode = true;
			bCptSimul = 0;
			bRestart=false;	
				
			return true;
		}
		else
		{
			debugdot("**Simulation : bCptSimul = %d",bCptSimul);
			bCptSimul++;
			
			return false;
		}
	}
	else {
		if( bLvLOneLoaded && bSimulation ) //remplacer par bSimulation
		{
		
		if( bCptSimul == 20 )
		{
			debugdot("**Simulation : bCptSimul = 20 : bLvlOneLoaded");
			
			etat = 0;
			bLvLOneLoaded = false;	
				
			bCptSimul = 0;
			return true;
	
		}
		else
		{
			bCptSimul++;
			debugdot("**Simulation : bCptSimul = %d",bCptSimul);
			return false;
		}
	}
	}
	//END
	
	return true;
}

void DefaultEventManager::pushEvent(const Common::Event &event) {
	// If already received an EVENT_QUIT, don't add another one
	debugdot("DefaultEventManager::PushEvent");
	if (event.type == Common::EVENT_QUIT) {
		if (!_shouldQuit)
			_artificialEventSource.addEvent(event);
	} else
		_artificialEventSource.addEvent(event);
}

#endif // !defined(DISABLE_DEFAULT_EVENTMANAGER)
